@@ -6,10 +6,10 @@ from ctypes import *
 import _ctypes
 from typing import Union, IO, List
 
-__version__ = '0.3.9'
+__version__ = '0.3.10'
 
-# experimental
-plot_library = 'matplotlib'  # 'plotly'
+# library to use in plot_result()
+plot_library = 'plotly'  # or 'matplotlib'
 
 
 # determine the platform
@@ -57,7 +57,7 @@ if sys.maxsize > 2**32:
     architecture = 'x86_64'
 else:
     platform += '32'
-    architecture = 'i686'
+    architecture = 'x86'
 
 platform_tuple = architecture + '-' + system
 
