@@ -229,7 +229,7 @@ def validate_result(result, reference, stop_time=None):
     return rel_out
 
 
-def create_plotly_figure(result, names=None, events=False, time_unit=None):
+def create_plotly_figure(result, names=None, events=False, time_unit=None, markers=False):
 
     import plotly.graph_objects as go
     from plotly.subplots import make_subplots
@@ -311,15 +311,21 @@ def create_plotly_figure(result, names=None, events=False, time_unit=None):
             y = y * display_unit.factor + display_unit.offset
             unit = display_unit.name
 
-        line = dict(color='#636efa', width=1)
+        args = dict(
+            x=time,
+            name=name,
+            line=dict(color='#636efa', width=1),
+            mode='lines+markers' if markers else None,
+            marker=dict(color='#636efa', size=5)
+        )
 
         if y.dtype in [np.float32, np.float64]:
-            trace = go.Scatter(x=time, y=y, name=name, line=line)
+            trace = go.Scatter(y=y, **args)
         elif y.dtype == bool:
-            trace = go.Scatter(x=time, y=y.astype(int), name=name, line=line, fill='tozeroy', fillcolor='rgba(0,0,255,0.1)', line_shape='hv')
+            trace = go.Scatter(y=y.astype(int), fill='tozeroy', fillcolor='rgba(0,0,255,0.1)', line_shape='hv', **args)
             fig['layout'][f'yaxis{i + 1}'].update(tickvals=[0, 1], ticktext=['false', 'true'], range=[-0.1, 1.1], fixedrange=True)
         else:
-            trace = go.Scatter(x=time, y=y, name=name, line=line, line_shape='hv')
+            trace = go.Scatter(y=y, line_shape='hv', **args)
 
         fig.add_trace(trace, row=i + 1, col=1)
 
@@ -330,22 +336,19 @@ def create_plotly_figure(result, names=None, events=False, time_unit=None):
             fig.add_vline(x=t_event, line={'color': '#fbe424', 'width': 1})
 
     fig['layout']['height'] = 160 * len(names) + 30 * max(0, 5 - len(names))
-    fig['layout']['margin']['t'] = 30
-    fig['layout']['margin']['b'] = 0
-    fig['layout']['margin']['r'] = 30
-    fig['layout']['plot_bgcolor'] = 'rgba(0,0,0,0)'
     fig['layout'][f'xaxis{len(names)}'].update(title=f'time [{time_unit}]')
 
-    axes_attrs = dict(showgrid=True, gridwidth=1, ticklen=0, gridcolor='LightGrey', linecolor='black', showline=True, zerolinewidth=1, zerolinecolor='LightGrey')
-    fig.update_xaxes(zeroline=True, **axes_attrs)
+    axes_attrs = dict(showgrid=True, gridwidth=1, ticklen=0, gridcolor='LightGrey', linecolor='black', showline=True,
+                      zerolinewidth=1, zerolinecolor='LightGrey')
+    fig.update_xaxes(range=(time[0], time[-1]), **axes_attrs)
     fig.update_yaxes(**axes_attrs)
 
-    fig.update_layout(showlegend=False)
+    fig.update_layout(showlegend=False, margin=dict(t=30, b=0, r=30), plot_bgcolor='rgba(0,0,0,0)')
 
     return fig
 
 
-def plot_result(result, reference=None, names=None, filename=None, window_title=None, events=False):
+def plot_result(result, reference=None, names=None, filename=None, window_title=None, events=False, markers=False):
     """ Plot a collection of time series.
 
     Parameters:
@@ -355,12 +358,13 @@ def plot_result(result, reference=None, names=None, filename=None, window_title=
         filename:     when provided the plot is saved as `filename` instead of showing the figure
         window_title: title for the figure window
         events:       draw vertical lines at events
+        markers:      show markers
     """
 
     from . import plot_library
 
     if plot_library == 'plotly':
-        figure = create_plotly_figure(result, names=names, events=events)
+        figure = create_plotly_figure(result, names=names, events=events, markers=markers)
         if filename is None:
             figure.show()
         else:
@@ -599,7 +603,7 @@ def download_test_file(fmi_version, fmi_type, tool_name, tool_version, model_nam
     url = 'https://github.com/modelica/fmi-cross-check/raw/master/fmus'
     url = '/'.join([url, fmi_version, fmi_type, platform, tool_name, tool_version, model_name, filename])
 
-    download_file(url)
+    return download_file(url)
 
 
 def fmu_info(filename: Union[str, IO], causalities: List[str] = ['input', 'output']) -> str:
@@ -682,21 +686,28 @@ Model Info
     return '\n'.join(l)
 
 
-def visual_studio_installation_path():
-    """ Get the VisualStudio 2017 installation path """
+def visual_studio_installation_paths(only_latest=False):
+    """ Get the installation paths for Visual Studio 2017+ """
 
-    try:
-        from subprocess import Popen, PIPE
-        import os
-        vswhere = '"' + os.environ['ProgramFiles(x86)'] + r'\Microsoft Visual Studio\Installer\vswhere.exe"'
-        command = vswhere +  ' -latest -products * -requires Microsoft.Component.MSBuild -property installationPath'
-        proc = Popen(command, stdout=PIPE)
-        output, _ = proc.communicate()
-        return output.decode('utf-8').strip()
-    except Exception as e:
-        pass  # do noting
+    paths = []
 
-    return None
+    for version in [17, 16, 15]:
+        try:
+            from subprocess import Popen, PIPE
+            import os
+            vswhere = rf'{os.environ["ProgramFiles(x86)"]}\Microsoft Visual Studio\Installer\vswhere.exe'
+            command = f'"{vswhere}" -products * -version [{version},{version + 1}) -property installationPath'
+            proc = Popen(command, stdout=PIPE)
+            output, _ = proc.communicate()
+            output = output.decode('utf-8').strip()
+            if output:
+                paths.append((version, output))
+                if only_latest:
+                    break
+        except Exception as e:
+            pass  # do noting
+
+    return paths
 
 
 def visual_c_versions():
@@ -705,32 +716,30 @@ def visual_c_versions():
     Returns: a sorted list of detected Visual C versions e.g. [90, 120, 140]
     """
 
-    versions = []
+    versions = set()
 
     # up to Visual Studio 2015
     for key in os.environ.keys():
         if key.upper().startswith('VS') and key.upper().endswith('COMNTOOLS'):
-            versions.append(int(key[len('VS'):-len('COMNTOOLS')]))
+            versions.add(int(key[len('VS'):-len('COMNTOOLS')]))
 
-    # Visual Studio 2017
-    installation_path = visual_studio_installation_path()
+    # Visual Studio from 2017
+    installation_paths = visual_studio_installation_paths()
 
-    if installation_path is not None:
-        if '2017' in installation_path:
-            versions.append(150)
-        if '2019' in installation_path:
-            versions.append(160)
+    for version, _ in installation_paths:
+        versions.add(version * 10)
 
     return sorted(versions)
 
 
-def compile_dll(model_description, sources_dir, compiler=None, target_platform=None):
+def compile_dll(model_description, sources_dir, compiler=None, target_platform=None, compiler_options=None):
     """ Compile the shared library
 
     Parameters:
-        sources_dir    directory that contains the FMU's source code
-        compiler       compiler to use (None: use Visual C on Windows, GCC otherwise)
-        platform       platform to compile for the binary for
+        sources_dir       directory that contains the FMU's source code
+        compiler          compiler to use (None: use Visual C on Windows, GCC otherwise)
+        platform          platform to compile for the binary for
+        compiler_options  custom compiler options
     """
 
     from . import platform, system
@@ -745,11 +754,12 @@ def compile_dll(model_description, sources_dir, compiler=None, target_platform=N
     if compiler is None:
         if target_platform in ['win32', 'win64', 'x86_64-windows']:
             compiler = 'vc'
+        elif target_platform in ['darwin64', 'x86_64-darwin', 'aarch64-darwin']:
+            compiler = 'clang'
         else:
             compiler = 'gcc'
 
     include_dir = os.path.join(os.path.dirname(__file__), 'c-code')
-    preprocessor_definitions = []
 
     source_files = []
 
@@ -765,6 +775,12 @@ def compile_dll(model_description, sources_dir, compiler=None, target_platform=N
 
     source_files += source_file_set.sourceFiles
 
+    preprocessor_definitions = []
+
+    if model_description.fmiVersion.startswith('3.0'):
+        preprocessor_definitions.append('FMI3_OVERRIDE_FUNCTION_PREFIX')
+        preprocessor_definitions.append('FMI3_FUNCTION_PREFIX=""')
+
     for definition in source_file_set.preprocessorDefinitions:
         literal = definition.name
         if definition.value is not None:
@@ -789,6 +805,9 @@ def compile_dll(model_description, sources_dir, compiler=None, target_platform=N
 
         definitions = ' '.join(f' /D{d}' for d in preprocessor_definitions)
 
+        if compiler_options is None:
+            compiler_options = '/Oy /Ob1 /Oi /LD'
+
         if len(vc_versions) == 0:
             raise Exception("No VisualStudio found")
 
@@ -796,12 +815,13 @@ def compile_dll(model_description, sources_dir, compiler=None, target_platform=N
         vc_version = vc_versions[-1]
 
         if vc_version < 150:
-            command = rf'call "%%VS{vc_version}COMNTOOLS%%\..\..\VC\vcvarsall.bat" {toolset}'
+            command = rf'call "%VS{vc_version}COMNTOOLS%\..\..\VC\vcvarsall.bat" {toolset}'
         else:
-            installation_path = visual_studio_installation_path()
+            installation_paths = visual_studio_installation_paths(only_latest=True)
+            _, installation_path = installation_paths[-1]
             command = rf'call "{installation_path}\VC\Auxiliary\Build\vcvarsall.bat" {toolset}'
 
-        command += f' && cl /LD /I. /I"{include_dir}" {definitions} /Fe{model_identifier} shlwapi.lib {sources}'
+        command += f' && cl {compiler_options} /I. /I"{include_dir}" {definitions} /Fe{model_identifier} shlwapi.lib {sources}'
 
     elif compiler == 'gcc':
 
@@ -812,34 +832,48 @@ def compile_dll(model_description, sources_dir, compiler=None, target_platform=N
             cc = 'gcc'
             target = model_identifier + '.so'
 
+            if compiler_options is None:
+                compiler_options = '-fPIC'
+
             if system == 'windows':
                 cc = 'wsl ' + cc
                 output = subprocess.check_output(f"wsl wslpath -a '{include_dir}'")
                 include_dir = output.decode('utf-8').strip()
 
-            command = f'{cc} -c -I. -I{include_dir} -fPIC {definitions} {sources}'
+            command = f'{cc} -c {compiler_options} -I. -I{include_dir} {definitions} {sources}'
             command += f' && {cc} -static-libgcc -shared -o{target} *.o -lm'
 
-        elif target_platform in ['darwin64', 'x86_64-darwin']:
+        else:
+            raise Exception(f'The target platform "{target_platform}" is not supported for the gcc compiler.')
 
-            target = model_identifier + '.dylib'
+    elif compiler == 'clang':
 
-            command = f'gcc -c -I. -I{include_dir} {definitions} {sources}'
-            command += f' && gcc -shared -o{target} *.o -lm'
+        definitions = ' '.join(f' -D{d}' for d in preprocessor_definitions)
+
+        if target_platform in ['darwin64', 'x86_64-darwin']:
+
+            target = f'{model_identifier}.dylib'
+
+            if compiler_options is None:
+                compiler_options = '-arch x86_64 -arch arm64'
+
+            command = f'clang -c {compiler_options} -I. -I{include_dir} {definitions} {sources}'
+            command += f' && clang -shared -arch x86_64 -arch arm64 -o{target} *.o -lm'
 
         else:
-            raise Exception("Unsupported target platform for selected compiler: '%s'" % compiler)
+            raise Exception(f'The target platform "{target_platform}" is not supported for the clang compiler.')
+
     else:
-        raise Exception("Unsupported compiler: '%s'" % compiler)
+        raise Exception(f'The compiler "{compiler}" is not supported.')
+
+    print(sources_dir)
+    print(command)
 
     cur_dir = os.getcwd()
     os.chdir(sources_dir)
     status = os.system(command)
     os.chdir(cur_dir)
 
-    print(sources_dir)
-    print(command)
-
     dll_path = os.path.join(sources_dir, target)
 
     if status != 0 or not os.path.isfile(dll_path):
@@ -848,17 +882,19 @@ def compile_dll(model_description, sources_dir, compiler=None, target_platform=N
     return str(dll_path)
 
 
-def compile_platform_binary(filename, output_filename=None, target_platform=None):
+def compile_platform_binary(filename, output_filename=None, target_platform=None, compiler_options=None):
     """ Compile the binary of an FMU for the current platform and add it to the FMU
 
     Parameters:
-        filename:         filename of the source code FMU
-        output_filename:  filename of the FMU with the compiled binary (None: overwrite existing FMU)
+        filename:          filename of the source code FMU
+        output_filename:   filename of the FMU with the compiled binary (None: overwrite existing FMU)
+        compiler_options:  custom compiler options
     """
 
     from . import read_model_description, extract, platform, platform_tuple
     import zipfile
     from shutil import copyfile, rmtree
+    from os.path import join, basename, relpath, exists, normpath, isfile, splitext
 
     unzipdir = extract(filename)
 
@@ -868,32 +904,38 @@ def compile_platform_binary(filename, output_filename=None, target_platform=None
         target_platform = platform if model_description.fmiVersion in ['1.0', '2.0'] else platform_tuple
 
     binary = compile_dll(model_description=model_description,
-                         sources_dir=os.path.join(unzipdir, 'sources'),
-                         target_platform=target_platform)
+                         sources_dir=join(unzipdir, 'sources'),
+                         target_platform=target_platform,
+                         compiler_options=compiler_options)
 
     unzipdir2 = extract(filename)
 
-    target_platform_dir = os.path.join(unzipdir2, 'binaries', target_platform)
+    target_platform_dir = join(unzipdir2, 'binaries', target_platform)
 
-    if not os.path.exists(target_platform_dir):
+    if not exists(target_platform_dir):
         os.makedirs(target_platform_dir)
 
-    copyfile(src=binary, dst=os.path.join(target_platform_dir, os.path.basename(binary)))
+    copyfile(src=binary, dst=join(target_platform_dir, basename(binary)))
+
+    debug_database = splitext(binary)[0] + '.pdb'
+
+    if isfile(debug_database):
+        copyfile(src=debug_database, dst=join(target_platform_dir, basename(debug_database)))
 
     if output_filename is None:
         output_filename = filename  # overwrite the existing archive
 
     # create a new archive from the existing files + compiled binary
     with zipfile.ZipFile(output_filename, 'w', zipfile.ZIP_DEFLATED) as zf:
-        base_path = os.path.normpath(unzipdir2)
+        base_path = normpath(unzipdir2)
         for dirpath, dirnames, filenames in os.walk(unzipdir2):
             for name in sorted(dirnames):
-                path = os.path.normpath(os.path.join(dirpath, name))
-                zf.write(path, os.path.relpath(path, base_path))
+                path = normpath(join(dirpath, name))
+                zf.write(path, relpath(path, base_path))
             for name in filenames:
-                path = os.path.normpath(os.path.join(dirpath, name))
-                if os.path.isfile(path):
-                    zf.write(path, os.path.relpath(path, base_path))
+                path = normpath(join(dirpath, name))
+                if isfile(path):
+                    zf.write(path, relpath(path, base_path))
 
     # clean up
     rmtree(unzipdir, ignore_errors=True)
@@ -901,81 +943,81 @@ def compile_platform_binary(filename, output_filename=None, target_platform=None
 
 
 def add_remoting(filename, host_platform, remote_platform):
+    """
+        win32 on win64 (SM)
+        linux64 on win64 (WSL + TCP)
+        win64 on linux64 (wine + TCP)
+    """
 
     from . import extract, read_model_description, supported_platforms
     from shutil import copyfile, rmtree
     import zipfile
-    import os
+    from os.path import join, isdir, isfile, normpath, relpath
 
     platforms = supported_platforms(filename)
+    current_dir = os.path.dirname(__file__)
 
-    if host_platform == 'win64' and remote_platform == 'win32':
-
-        if 'win64' in platforms:
-            raise Exception('The FMU already supports "win64".')
-
-        if 'win32' not in platforms:
-            raise Exception('The FMU does not support the platform "win32".')
-
-    elif host_platform == 'linux64' and remote_platform == 'win64':
-
-        if 'linux64' in platforms:
-            raise Exception('The FMU already supports "linux64".')
-
-        if 'win64' not in platforms:
-            raise Exception('The FMU does not support the platform "win64".')
-    else:
+    methods = {
+        ('win64', 'win32'): 'sm',
+        ('win64', 'linux64'): 'tcp',
+        ('linux64', 'win64'): 'tcp'
+    }
 
+    if (host_platform, remote_platform) not in methods:
         raise Exception("Remoting is not supported for the given combination of host and remote platform.")
 
+    if host_platform in platforms:
+        raise Exception(f"The FMU already supports {host_platform}.")
+
+    if remote_platform not in platforms:
+        raise Exception(f"The FMU does not support {remote_platform}.")
+
+    method = methods[(host_platform, remote_platform)]
+
     model_description = read_model_description(filename)
 
-    current_dir = os.path.dirname(__file__)
+    license = join(current_dir, 'remoting', 'license.txt')
 
-    if host_platform == 'win64':
-        client = os.path.join(current_dir, 'remoting', 'win64', 'client.dll')
-        server = os.path.join(current_dir, 'remoting', 'win32', 'server.exe')
+    if isdir(filename):
+        tempdir = filename
     else:
-        client = os.path.join(current_dir, 'remoting', 'linux64', 'client.so')
-        server = os.path.join(current_dir, 'remoting', 'win64', 'server.exe')
-
-    license = os.path.join(current_dir, 'remoting', 'license.txt')
-
-    tempdir = extract(filename)
+        tempdir = extract(filename)
 
     if model_description.coSimulation:
         model_identifier = model_description.coSimulation.modelIdentifier
     else:
         model_identifier = model_description.modelExchange.modelIdentifier
 
+    sl_ext = {'linux64': '.so', 'win32': '.dll', 'win64': '.dll'}
+    ex_ext = {'linux64': '', 'win32': '.exe', 'win64': '.exe'}
+
     # copy the binaries & license
-    if host_platform == 'win64':
-        os.mkdir(os.path.join(tempdir, 'binaries', 'win64'))
-        copyfile(client, os.path.join(tempdir, 'binaries', 'win64', model_identifier + '.dll'))
-        copyfile(server, os.path.join(tempdir, 'binaries', 'win32', 'server.exe'))
-    else:
-        os.mkdir(os.path.join(tempdir, 'binaries', 'linux64'))
-        copyfile(client, os.path.join(tempdir, 'binaries', 'linux64', model_identifier + '.so'))
-        copyfile(server, os.path.join(tempdir, 'binaries', 'win64', 'server.exe'))
+    os.makedirs(join(tempdir, 'binaries', host_platform), exist_ok=True)
 
-    licenses_dir = os.path.join(tempdir, 'documentation', 'licenses')
+    copyfile(src=join(current_dir, 'remoting', host_platform, f'client_{method}{sl_ext[host_platform]}'),
+             dst=join(tempdir, 'binaries', host_platform, model_identifier + sl_ext[host_platform]))
+
+    copyfile(src=join(current_dir, 'remoting', remote_platform, f'server_{method}{ex_ext[remote_platform]}'),
+             dst=join(tempdir, 'binaries', remote_platform, f'server_{method}{ex_ext[remote_platform]}'))
+
+    licenses_dir = join(tempdir, 'documentation', 'licenses')
     os.makedirs(licenses_dir, exist_ok=True)
-    copyfile(license, os.path.join(tempdir, 'documentation', 'licenses', 'fmpy-remoting-binaries.txt'))
+    copyfile(license, join(tempdir, 'documentation', 'licenses', 'fmpy-remoting-binaries.txt'))
 
-    # create a new archive from the existing files + remoting binaries
-    with zipfile.ZipFile(filename, 'w', zipfile.ZIP_DEFLATED) as zf:
-        base_path = os.path.normpath(tempdir)
-        for dirpath, dirnames, filenames in os.walk(tempdir):
-            for name in sorted(dirnames):
-                path = os.path.normpath(os.path.join(dirpath, name))
-                zf.write(path, os.path.relpath(path, base_path))
-            for name in filenames:
-                path = os.path.normpath(os.path.join(dirpath, name))
-                if os.path.isfile(path):
-                    zf.write(path, os.path.relpath(path, base_path))
-
-    # clean up
-    rmtree(tempdir, ignore_errors=True)
+    if not isdir(filename):
+        # create a new archive from the existing files + remoting binaries
+        with zipfile.ZipFile(filename, 'w', zipfile.ZIP_DEFLATED) as zf:
+            base_path = normpath(tempdir)
+            for dirpath, dirnames, filenames in os.walk(tempdir):
+                for name in sorted(dirnames):
+                    path = normpath(join(dirpath, name))
+                    zf.write(path, relpath(path, base_path))
+                for name in filenames:
+                    path = normpath(join(dirpath, name))
+                    if isfile(path):
+                        zf.write(path, relpath(path, base_path))
+        # clean up
+        rmtree(tempdir, ignore_errors=True)
 
 
 def auto_interval(t):
